@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <arpa/inet.h>
 
 #include <usbmuxd.h>
 #include <gnutls/gnutls.h>
@@ -160,7 +159,7 @@ idevice_error_t idevice_new(idevice_t * device, const char *uuid)
 	usbmuxd_device_info_t muxdev;
 	int res = usbmuxd_get_device_by_uuid(uuid, &muxdev);
 	if (res > 0) {
-		idevice_t phone = (idevice_t) malloc(sizeof(struct idevice_int));
+		idevice_t phone = (idevice_t) malloc(sizeof(struct idevice_private));
 		phone->uuid = strdup(muxdev.uuid);
 		phone->conn_type = CONNECTION_USBMUXD;
 		phone->conn_data = (void*)muxdev.handle;
@@ -172,7 +171,8 @@ idevice_error_t idevice_new(idevice_t * device, const char *uuid)
 	return IDEVICE_E_NO_DEVICE;
 }
 
-/** Cleans up an idevice structure, then frees the structure itself.  
+/**
+ * Cleans up an idevice structure, then frees the structure itself.
  * This is a library-level function; deals directly with the device to tear
  *  down relations, but otherwise is mostly internal.
  * 
@@ -220,7 +220,7 @@ idevice_error_t idevice_connect(idevice_t device, uint16_t port, idevice_connect
 			debug_info("ERROR: Connecting to usbmuxd failed: %d (%s)", sfd, strerror(-sfd));
 			return IDEVICE_E_UNKNOWN_ERROR;
 		}
-		idevice_connection_t new_connection = (idevice_connection_t)malloc(sizeof(struct idevice_connection_int));
+		idevice_connection_t new_connection = (idevice_connection_t)malloc(sizeof(struct idevice_connection_private));
 		new_connection->type = CONNECTION_USBMUXD;
 		new_connection->data = (void*)sfd;
 		new_connection->ssl_data = NULL;
@@ -422,6 +422,9 @@ idevice_error_t idevice_connection_receive(idevice_connection_t connection, char
 	return internal_connection_receive(connection, data, len, recv_bytes);
 }
 
+/**
+ * Gets the handle of the device. Depends on the connection type.
+ */
 idevice_error_t idevice_get_handle(idevice_t device, uint32_t *handle)
 {
 	if (!device)
@@ -436,6 +439,9 @@ idevice_error_t idevice_get_handle(idevice_t device, uint32_t *handle)
 	return IDEVICE_E_UNKNOWN_ERROR;
 }
 
+/**
+ * Gets the unique id for the device.
+ */
 idevice_error_t idevice_get_uuid(idevice_t device, char **uuid)
 {
 	if (!device)
@@ -469,10 +475,10 @@ static ssize_t internal_ssl_read(gnutls_transport_ptr_t transport, char *buffer,
 		}
 		debug_info("post-read we got %i bytes", bytes);
 
-		// increase read count
+		/* increase read count */
 		tbytes += bytes;
 
-		// fill the buffer with what we got right now
+		/* fill the buffer with what we got right now */
 		memcpy(buffer + pos_start_fill, recv_buffer, bytes);
 		pos_start_fill += bytes;
 
@@ -536,9 +542,9 @@ idevice_error_t idevice_connection_enable_ssl(idevice_connection_t connection)
 	idevice_error_t ret = IDEVICE_E_SSL_ERROR;
 	uint32_t return_me = 0;
 
-	ssl_data_t ssl_data_loc = (ssl_data_t)malloc(sizeof(struct ssl_data_int));
+	ssl_data_t ssl_data_loc = (ssl_data_t)malloc(sizeof(struct ssl_data_private));
 
-	// Set up GnuTLS...
+	/* Set up GnuTLS... */
 	debug_info("enabling SSL mode");
 	errno = 0;
 	gnutls_global_init();
@@ -558,7 +564,7 @@ idevice_error_t idevice_connection_enable_ssl(idevice_connection_t connection)
 		gnutls_protocol_set_priority(ssl_data_loc->session, protocol_priority);
 		gnutls_mac_set_priority(ssl_data_loc->session, mac_priority);
 	}
-	gnutls_credentials_set(ssl_data_loc->session, GNUTLS_CRD_CERTIFICATE, ssl_data_loc->certificate); // this part is killing me.
+	gnutls_credentials_set(ssl_data_loc->session, GNUTLS_CRD_CERTIFICATE, ssl_data_loc->certificate); /* this part is killing me. */
 
 	debug_info("GnuTLS step 1...");
 	gnutls_transport_set_ptr(ssl_data_loc->session, (gnutls_transport_ptr_t)connection);
