@@ -1,6 +1,6 @@
 /*
  * installation_proxy.c
- * Installation Proxy implementation.
+ * com.apple.mobile.installation_proxy service implementation.
  *
  * Copyright (c) 2009 Nikias Bassen, All Rights Reserved.
  *
@@ -22,7 +22,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <arpa/inet.h>
 #include <plist/plist.h>
 
 #include "installation_proxy.h"
@@ -35,7 +34,8 @@ struct instproxy_status_data {
 	char *operation;
 };
 
-/** Locks an installation_proxy client, done for thread safety stuff.
+/**
+ * Locks an installation_proxy client, used for thread safety.
  *
  * @param client The installation_proxy client to lock
  */
@@ -45,7 +45,8 @@ static void instproxy_lock(instproxy_client_t client)
 	g_mutex_lock(client->mutex);
 }
 
-/** Unlocks an installation_proxy client, done for thread safety stuff.
+/**
+ * Unlocks an installation_proxy client, used for thread safety.
  * 
  * @param client The installation_proxy client to lock
  */
@@ -82,7 +83,7 @@ static instproxy_error_t instproxy_error(property_list_service_error_t err)
 }
 
 /**
- * Creates a new installation_proxy client
+ * Connects to the installation_proxy service on the specified device.
  *
  * @param device The device to connect to
  * @param port Destination port (usually given by lockdownd_start_service).
@@ -106,7 +107,7 @@ instproxy_error_t instproxy_client_new(idevice_t device, uint16_t port, instprox
 		return INSTPROXY_E_CONN_FAILED;
 	}
 
-	instproxy_client_t client_loc = (instproxy_client_t) malloc(sizeof(struct instproxy_client_int));
+	instproxy_client_t client_loc = (instproxy_client_t) malloc(sizeof(struct instproxy_client_private));
 	client_loc->parent = plistclient;
 	client_loc->mutex = g_mutex_new();
 	client_loc->status_updater = NULL;
@@ -116,9 +117,10 @@ instproxy_error_t instproxy_client_new(idevice_t device, uint16_t port, instprox
 }
 
 /**
- * Frees an installation_proxy client.
+ * Disconnects an installation_proxy client from the device and frees up the
+ * installation_proxy client data.
  *
- * @param client The installation_proxy client to free.
+ * @param client The installation_proxy client to disconnect and free.
  *
  * @return INSTPROXY_E_SUCCESS on success
  *      or INSTPROXY_E_INVALID_ARG if client is NULL.
@@ -144,7 +146,7 @@ instproxy_error_t instproxy_client_free(instproxy_client_t client)
 
 /**
  * Send a command with specified options to the device.
- * Internally used only.
+ * Only used internally.
  *
  * @param client The connected installation_proxy client.
  * @param command The command to execute. Required.
@@ -737,21 +739,21 @@ void instproxy_client_options_add(plist_t client_options, ...)
 		char *key = strdup(arg);
 		if (!strcmp(key, "SkipUninstall")) {
 			int intval = va_arg(args, int);
-			plist_dict_set_item(client_options, key, plist_new_bool(intval));
+			plist_dict_insert_item(client_options, key, plist_new_bool(intval));
 		} else if (!strcmp(key, "ApplicationSINF") || !strcmp(key, "iTunesMetadata")) {
 			plist_t plistval = va_arg(args, plist_t);
 			if (!plistval) {
 				free(key);
 				break;
 			}
-			plist_dict_set_item(client_options, key, plist_copy(plistval));
+			plist_dict_insert_item(client_options, key, plist_copy(plistval));
 		} else {
 			char *strval = va_arg(args, char*);
 			if (!strval) {
 				free(key);
 				break;
 			}
-			plist_dict_set_item(client_options, key, plist_new_string(strval));
+			plist_dict_insert_item(client_options, key, plist_new_string(strval));
 		}
 		free(key);
 		arg = va_arg(args, char*);
