@@ -59,7 +59,7 @@ enum device_link_file_status_t {
 	DEVICE_LINK_FILE_STATUS_LAST_HUNK
 };
 
-static void notify_cb(const char *notification)
+static void notify_cb(const char *notification, void *userdata)
 {
 	if (!strcmp(notification, NP_SYNC_CANCEL_REQUEST)) {
 		printf("User has aborted on-device\n");
@@ -150,17 +150,26 @@ static void buffer_read_from_filename(const char *filename, char **buffer, uint3
 	FILE *f;
 	uint64_t size;
 
+	*length = 0;
+
 	f = fopen(filename, "rb");
+	if (!f) {
+		return;
+	}
 
 	fseek(f, 0, SEEK_END);
 	size = ftell(f);
 	rewind(f);
 
+	if (size == 0) {
+		return;
+	}
+
 	*buffer = (char*)malloc(sizeof(char)*size);
 	fread(*buffer, sizeof(char), size, f);
 	fclose(f);
 
-	*length = size;
+	*length = (uint32_t)size;
 }
 
 static void buffer_write_to_filename(const char *filename, const char *buffer, uint32_t length)
@@ -186,7 +195,7 @@ static int plist_read_from_filename(plist_t *plist, const char *filename)
 		return 0;
 	}
 
-	if (memcmp(buffer, "bplist00", 8) == 0) {
+	if ((length > 8) && (memcmp(buffer, "bplist00", 8) == 0)) {
 		plist_from_bin(buffer, length, plist);
 	} else {
 		plist_from_xml(buffer, length, plist);
@@ -234,35 +243,6 @@ static int plist_strcmp(plist_t node, const char *str)
 	return ret;
 }
 
-static plist_t device_link_message_factory_process_message_new(plist_t content)
-{
-	plist_t ret = plist_new_array();
-	plist_array_append_item(ret, plist_new_string("DLMessageProcessMessage"));
-	plist_array_append_item(ret, content);
-	return ret;
-}
-
-static void mobilebackup_cancel_backup_with_error(const char *reason)
-{
-	plist_t node = plist_new_dict();
-	plist_dict_insert_item(node, "BackupMessageTypeKey", plist_new_string("BackupMessageError"));
-	plist_dict_insert_item(node, "BackupErrorReasonKey", plist_new_string(reason));
-
-	plist_t message = device_link_message_factory_process_message_new(node);
-
-	mobilebackup_send(mobilebackup, message);
-
-	plist_free(message);
-	message = NULL;
-}
-
-static plist_t mobilebackup_factory_backup_file_received_new()
-{
-	plist_t node = plist_new_dict();
-	plist_dict_insert_item(node, "BackupMessageTypeKey", plist_new_string("kBackupMessageBackupFileReceived"));
-	return device_link_message_factory_process_message_new(node);
-}
-
 static gchar *mobilebackup_build_path(const char *backup_directory, const char *name, const char *extension)
 {
 	gchar *filename = g_strconcat(name, extension, NULL);
@@ -543,7 +523,7 @@ int main(int argc, char *argv[])
 	ret = lockdownd_start_service(client, NP_SERVICE_NAME, &port);
 	if ((ret == LOCKDOWN_E_SUCCESS) && port) {
 		np_client_new(phone, port, &np);
-		np_set_notify_callback(np, notify_cb);
+		np_set_notify_callback(np, notify_cb, NULL);
 		const char *noties[5] = {
 			NP_SYNC_CANCEL_REQUEST,
 			NP_SYNC_SUSPEND_REQUEST,
@@ -582,7 +562,11 @@ int main(int argc, char *argv[])
 			printf("Reading Info.plist from backup.\n");
 			plist_read_from_filename(&info_plist, info_path);
 
-			if (cmd == CMD_BACKUP) {
+			if (!info_plist) {
+				printf("Could not read Info.plist\n");
+				is_full_backup = 1;
+			}
+			if (info_plist && (cmd == CMD_BACKUP)) {
 				if (mobilebackup_info_is_current_device(info_plist)) {
 					/* update the last backup time within Info.plist */
 					mobilebackup_info_update_last_backup_date(info_plist);
@@ -616,16 +600,6 @@ int main(int argc, char *argv[])
 			/* TODO: check domain com.apple.mobile.backup key RequiresEncrypt and WillEncrypt with lockdown */
 			/* TODO: verify battery on AC enough battery remaining */	
 
-			/* Info.plist (Device infos, IC-Info.sidb, photos, app_ids, iTunesPrefs) */
-			/* create new Info.plist on new backups */
-			if (is_full_backup) {
-				printf("Creating Info.plist for new backup.\n");
-				info_plist = mobilebackup_factory_info_plist_new();
-				plist_write_to_filename(info_plist, info_path, PLIST_FORMAT_XML);
-			}
-
-			g_free(info_path);
-
 			/* Manifest.plist (backup manifest (backup state)) */
 			char *manifest_path = mobilebackup_build_path(backup_directory, "Manifest", ".plist");
 
@@ -633,8 +607,27 @@ int main(int argc, char *argv[])
 			if (!is_full_backup) {
 				printf("Reading existing Manifest.\n");
 				plist_read_from_filename(&manifest_plist, manifest_path);
+				if (!manifest_plist) {
+					printf("Could not read Manifest.plist, switching to full backup mode.\n");
+					is_full_backup = 1;
+				}
 			}
 
+			/* Info.plist (Device infos, IC-Info.sidb, photos, app_ids, iTunesPrefs) */
+
+			/* create new Info.plist on new backups */
+			if (is_full_backup) {
+				if (info_plist) {
+					plist_free(info_plist);
+					info_plist = NULL;
+				}
+				remove(info_path);
+				printf("Creating Info.plist for new backup.\n");
+				info_plist = mobilebackup_factory_info_plist_new();
+				plist_write_to_filename(info_plist, info_path, PLIST_FORMAT_XML);
+			}
+			g_free(info_path);
+
 			plist_free(info_plist);
 			info_plist = NULL;
 
@@ -650,54 +643,27 @@ int main(int argc, char *argv[])
 			/* request backup from device with manifest from last backup */
 			printf("Requesting backup from device...\n");
 
-			node = plist_new_dict();
-
-			if (manifest_plist)
-				plist_dict_insert_item(node, "BackupManifestKey", manifest_plist);
-
-			plist_dict_insert_item(node, "BackupComputerBasePathKey", plist_new_string("/"));
-			plist_dict_insert_item(node, "BackupMessageTypeKey", plist_new_string("BackupMessageBackupRequest"));
-			plist_dict_insert_item(node, "BackupProtocolVersion", plist_new_string("1.6"));
-
-			plist_t message = device_link_message_factory_process_message_new(node);
-			mobilebackup_send(mobilebackup, message);
-			plist_free(message);
-			message = NULL;
-			node = NULL;
-
-			/* get response */
-			int backup_ok = 0;
-			mobilebackup_receive(mobilebackup, &message);
-
-			node = plist_array_get_item(message, 0);
-			if (!plist_strcmp(node, "DLMessageProcessMessage")) {
-				node = plist_array_get_item(message, 1);
-				node = plist_dict_get_item(node, "BackupMessageTypeKey");
-				if (node && !plist_strcmp(node, "BackupMessageBackupReplyOK")) {
-					printf("Device accepts manifest and will send backup data now...\n");
-					backup_ok = 1;
-					printf("Acknowledging...\n");
-					if (is_full_backup)
-						printf("Full backup mode.\n");
-					else
-						printf("Incremental backup mode.\n");
-					printf("Please wait. Device prepares backup data...\n");
-					/* send it back for ACK */
-					mobilebackup_send(mobilebackup, message);
-				}
+			mobilebackup_error_t err = mobilebackup_request_backup(mobilebackup, manifest_plist, "/", "1.6");
+			if (err == MOBILEBACKUP_E_SUCCESS) {
+				if (is_full_backup)
+					printf("Full backup mode.\n");
+				else
+					printf("Incremental backup mode.\n");
+				printf("Please wait. Device is preparing backup data...\n");
 			} else {
-				printf("ERROR: Unhandled message received!\n");
-			}
-			plist_free(message);
-			message = NULL;
-
-			if (!backup_ok) {
-				printf("ERROR: Device rejected to start the backup process.\n");
+				if (err == MOBILEBACKUP_E_BAD_VERSION) {
+					printf("ERROR: Could not start backup process: backup protocol version mismatch!\n");
+				} else if (err == MOBILEBACKUP_E_REPLY_NOT_OK) {
+					printf("ERROR: Could not start backup process: device refused to start the backup process.\n");
+				} else {
+					printf("ERROR: Could not start backup process: unspecified error occured\n");
+				}
 				break;
 			}
 
 			/* reset backup status */
-			backup_ok = 0;
+			int backup_ok = 0;
+			plist_t message = NULL;
 
 			/* receive and save DLSendFile files and metadata, ACK each */
 			int file_index = 0;
@@ -836,15 +802,12 @@ int main(int argc, char *argv[])
 						printf("DONE\n");
 
 					/* acknowlegdge that we received the file */
-					message = mobilebackup_factory_backup_file_received_new();
-					mobilebackup_send(mobilebackup, message);
-					plist_free(message);
-					message = NULL;
+					mobilebackup_send_backup_file_received(mobilebackup);
 				}
 
 				if (quit_flag > 0) {
 					/* need to cancel the backup here */
-					mobilebackup_cancel_backup_with_error("Cancelling DLSendFile");
+					mobilebackup_send_error(mobilebackup, "Cancelling DLSendFile");
 
 					/* remove any atomic Manifest.plist.tmp */
 					if (manifest_path)
